"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getTitaniumDeviceAuthorizationData = exports.createTitaniumHeaders = exports.createTitaniumDeviceHeader = exports.createTitaniumAuthHeader = exports.createErrorForMalformedTokenConfiguration = exports.createErrorForMalformedDeviceInfoConfiguration = exports.TitaniumCDMType = exports.TITANIUM_CDM_DESCRIPTIONS = void 0;

var _reactNativeTheoplayer = require("@wouterds/react-native-theoplayer");

var _reactNativeDeviceInfo = require("react-native-device-info");

var _TitaniumUtils = require("./TitaniumUtils");

let TitaniumCDMType;
exports.TitaniumCDMType = TitaniumCDMType;

(function (TitaniumCDMType) {
  TitaniumCDMType["WIDEVINE"] = "Widevine";
  TitaniumCDMType["PLAYREADY"] = "PlayreadyV2";
  TitaniumCDMType["PLAYREADY_v2"] = "PlayreadyV2";
  TitaniumCDMType["PLAYREADY_v3"] = "PlayreadyV3";
  TitaniumCDMType["FAIRPLAY"] = "Fairplay";
})(TitaniumCDMType || (exports.TitaniumCDMType = TitaniumCDMType = {}));

const TITANIUM_CDM_DESCRIPTIONS = {
  Widevine: {
    DRMProvider: 'Google',
    DRMVersion: '1.4.8.86',
    DRMType: 'Widevine'
  },
  PlayreadyV2: {
    DRMProvider: 'Microsoft',
    DRMVersion: '2.9',
    DRMType: 'Playready'
  },
  PlayreadyV3: {
    DRMProvider: 'Microsoft',
    DRMVersion: '3',
    DRMType: 'Playready'
  },
  Fairplay: {
    DRMProvider: 'Apple',
    DRMType: 'FairPlay',
    DRMVersion: '1.0'
  }
};
exports.TITANIUM_CDM_DESCRIPTIONS = TITANIUM_CDM_DESCRIPTIONS;

const getTitaniumDeviceAuthorizationData = (integrationParameters, cdmType) => {
  const cdmDescription = TITANIUM_CDM_DESCRIPTIONS[cdmType];
  const accountName = integrationParameters.accountName;
  const customerName = integrationParameters.customerName;
  const portalId = integrationParameters.portalId;
  const friendlyName = integrationParameters.friendlyName;
  const vendor = (0, _reactNativeDeviceInfo.getManufacturerSync)();
  return {
    LatensRegistration: {
      CustomerName: customerName,
      AccountName: accountName,
      PortalId: portalId,
      FriendlyName: friendlyName,
      AppVersion: (0, _reactNativeDeviceInfo.getReadableVersion)(),
      DeviceInfo: {
        FormatVersion: '1',
        DeviceType: (0, _reactNativeDeviceInfo.getDeviceType)(),
        OSType: (0, _reactNativeDeviceInfo.getSystemName)(),
        OSVersion: (0, _reactNativeDeviceInfo.getSystemVersion)(),
        DRMProvider: cdmDescription.DRMProvider,
        DRMVersion: cdmDescription.DRMVersion,
        DRMType: cdmDescription.DRMType,
        DeviceVendor: vendor,
        DeviceModel: (0, _reactNativeDeviceInfo.getModel)()
      }
    }
  };
};

exports.getTitaniumDeviceAuthorizationData = getTitaniumDeviceAuthorizationData;

const createErrorForMalformedDeviceInfoConfiguration = (configuration, cdmType) => {
  let message = `Invalid Titanium ${cdmType} DRM configuration.`;

  if (!configuration.integrationParameters.accountName) {
    message = `Invalid Titanium ${cdmType} DRM configuration, accountName is not set.`;
  } else if (!configuration.integrationParameters.customerName) {
    message = `Invalid Titanium ${cdmType} DRM configuration, customerName is not set.`;
  } else if (!configuration.integrationParameters.portalId) {
    message = `Invalid Titanium ${cdmType} DRM configuration, portalId is not set.`;
  }

  throw {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    code: ErrorCode.CONTENT_PROTECTION_CONFIGURATION_INVALID,
    message: message
  };
};

exports.createErrorForMalformedDeviceInfoConfiguration = createErrorForMalformedDeviceInfoConfiguration;

const createErrorForMalformedTokenConfiguration = (configuration, cdmType) => {
  let message = `Invalid Titanium ${cdmType} DRM configuration.`;

  if (!configuration.integrationParameters.authToken) {
    message = `Invalid Titanium ${cdmType} DRM configuration, authToken is not set.`;
  }

  throw {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    code: ErrorCode.CONTENT_PROTECTION_CONFIGURATION_INVALID,
    message: message
  };
};

exports.createErrorForMalformedTokenConfiguration = createErrorForMalformedTokenConfiguration;

const createTitaniumAuthHeader = (configuration, cdmType) => {
  if ((0, _TitaniumUtils.isTokenBasedTitaniumDRMConfiguration)(configuration.integrationParameters)) {
    return `Bearer ${configuration.integrationParameters.authToken}`;
  } else {
    throw createErrorForMalformedTokenConfiguration(configuration, cdmType);
  }
};

exports.createTitaniumAuthHeader = createTitaniumAuthHeader;

const createTitaniumDeviceHeader = (configuration, cdmType) => {
  if ((0, _TitaniumUtils.isDeviceBasedTitaniumDRMConfiguration)(configuration.integrationParameters)) {
    const conf = configuration.integrationParameters;
    const deviceAuthorizationData = getTitaniumDeviceAuthorizationData(conf, cdmType);
    return (0, _reactNativeTheoplayer.fromObjectToBase64String)(deviceAuthorizationData);
  } else {
    throw createErrorForMalformedDeviceInfoConfiguration(configuration, cdmType);
  }
};

exports.createTitaniumDeviceHeader = createTitaniumDeviceHeader;

const createTitaniumHeaders = (configuration, cdmType) => {
  const hasAuthToken = (0, _TitaniumUtils.isTokenBasedTitaniumDRMConfiguration)(configuration.integrationParameters);

  if (hasAuthToken) {
    return {
      'content-type': 'application/octet-stream',
      Authorization: createTitaniumAuthHeader(configuration, cdmType)
    };
  } else {
    return {
      'content-type': 'application/octet-stream',
      'X-TITANIUM-DRM-CDATA': createTitaniumDeviceHeader(configuration, cdmType)
    };
  }
};

exports.createTitaniumHeaders = createTitaniumHeaders;
//# sourceMappingURL=TitaniumBaseRegistration.js.map