import { fromObjectToBase64String } from '@wouterds/react-native-theoplayer';
import { getDeviceType, getManufacturerSync, getModel, getReadableVersion, getSystemName, getSystemVersion } from 'react-native-device-info';
import { isDeviceBasedTitaniumDRMConfiguration, isTokenBasedTitaniumDRMConfiguration } from './TitaniumUtils';
export let TitaniumCDMType;

(function (TitaniumCDMType) {
  TitaniumCDMType["WIDEVINE"] = "Widevine";
  TitaniumCDMType["PLAYREADY"] = "PlayreadyV2";
  TitaniumCDMType["PLAYREADY_v2"] = "PlayreadyV2";
  TitaniumCDMType["PLAYREADY_v3"] = "PlayreadyV3";
  TitaniumCDMType["FAIRPLAY"] = "Fairplay";
})(TitaniumCDMType || (TitaniumCDMType = {}));

export const TITANIUM_CDM_DESCRIPTIONS = {
  Widevine: {
    DRMProvider: 'Google',
    DRMVersion: '1.4.8.86',
    DRMType: 'Widevine'
  },
  PlayreadyV2: {
    DRMProvider: 'Microsoft',
    DRMVersion: '2.9',
    DRMType: 'Playready'
  },
  PlayreadyV3: {
    DRMProvider: 'Microsoft',
    DRMVersion: '3',
    DRMType: 'Playready'
  },
  Fairplay: {
    DRMProvider: 'Apple',
    DRMType: 'FairPlay',
    DRMVersion: '1.0'
  }
};
export const getTitaniumDeviceAuthorizationData = (integrationParameters, cdmType) => {
  const cdmDescription = TITANIUM_CDM_DESCRIPTIONS[cdmType];
  const accountName = integrationParameters.accountName;
  const customerName = integrationParameters.customerName;
  const portalId = integrationParameters.portalId;
  const friendlyName = integrationParameters.friendlyName;
  const vendor = getManufacturerSync();
  return {
    LatensRegistration: {
      CustomerName: customerName,
      AccountName: accountName,
      PortalId: portalId,
      FriendlyName: friendlyName,
      AppVersion: getReadableVersion(),
      DeviceInfo: {
        FormatVersion: '1',
        DeviceType: getDeviceType(),
        OSType: getSystemName(),
        OSVersion: getSystemVersion(),
        DRMProvider: cdmDescription.DRMProvider,
        DRMVersion: cdmDescription.DRMVersion,
        DRMType: cdmDescription.DRMType,
        DeviceVendor: vendor,
        DeviceModel: getModel()
      }
    }
  };
};
export const createErrorForMalformedDeviceInfoConfiguration = (configuration, cdmType) => {
  let message = `Invalid Titanium ${cdmType} DRM configuration.`;

  if (!configuration.integrationParameters.accountName) {
    message = `Invalid Titanium ${cdmType} DRM configuration, accountName is not set.`;
  } else if (!configuration.integrationParameters.customerName) {
    message = `Invalid Titanium ${cdmType} DRM configuration, customerName is not set.`;
  } else if (!configuration.integrationParameters.portalId) {
    message = `Invalid Titanium ${cdmType} DRM configuration, portalId is not set.`;
  }

  throw {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    code: ErrorCode.CONTENT_PROTECTION_CONFIGURATION_INVALID,
    message: message
  };
};
export const createErrorForMalformedTokenConfiguration = (configuration, cdmType) => {
  let message = `Invalid Titanium ${cdmType} DRM configuration.`;

  if (!configuration.integrationParameters.authToken) {
    message = `Invalid Titanium ${cdmType} DRM configuration, authToken is not set.`;
  }

  throw {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    code: ErrorCode.CONTENT_PROTECTION_CONFIGURATION_INVALID,
    message: message
  };
};
export const createTitaniumAuthHeader = (configuration, cdmType) => {
  if (isTokenBasedTitaniumDRMConfiguration(configuration.integrationParameters)) {
    return `Bearer ${configuration.integrationParameters.authToken}`;
  } else {
    throw createErrorForMalformedTokenConfiguration(configuration, cdmType);
  }
};
export const createTitaniumDeviceHeader = (configuration, cdmType) => {
  if (isDeviceBasedTitaniumDRMConfiguration(configuration.integrationParameters)) {
    const conf = configuration.integrationParameters;
    const deviceAuthorizationData = getTitaniumDeviceAuthorizationData(conf, cdmType);
    return fromObjectToBase64String(deviceAuthorizationData);
  } else {
    throw createErrorForMalformedDeviceInfoConfiguration(configuration, cdmType);
  }
};
export const createTitaniumHeaders = (configuration, cdmType) => {
  const hasAuthToken = isTokenBasedTitaniumDRMConfiguration(configuration.integrationParameters);

  if (hasAuthToken) {
    return {
      'content-type': 'application/octet-stream',
      Authorization: createTitaniumAuthHeader(configuration, cdmType)
    };
  } else {
    return {
      'content-type': 'application/octet-stream',
      'X-TITANIUM-DRM-CDATA': createTitaniumDeviceHeader(configuration, cdmType)
    };
  }
};
//# sourceMappingURL=TitaniumBaseRegistration.js.map